from typing import Any, Type

from httpx import AsyncClient

from pydantic_client.clients.abstract_client import AbstractClient
from pydantic_client.proxy import AsyncClientProxy, Proxy
from pydantic_client.schema.http_request import HttpRequest


class HttpxClient(AbstractClient):
    runner_class: Type[Proxy] = AsyncClientProxy

    def __init__(self, base_url: str, http2: bool = False):
        self.base_url = base_url
        self.http2 = http2

    async def do_request(self, request: HttpRequest) -> Any:
        data, json = self.parse_request(request)
        async with AsyncClient(http2=self.http2) as session:
            try:
                response = await session.request(
                    url=self.base_url + request.url,
                    method=request.method,
                    json=json,
                    data=data
                )
                response.raise_for_status()
                if response.is_success:
                    return response.json()
            except BaseException as e:
                raise e



from functools import partial
from typing import Callable

from pydantic_client.schema.method_info import MethodInfo
from pydantic_client.utility import parse_func


def rest(
    url,
    method: str = "GET",
    form_body: bool = False
) -> Callable:
    def wrapper(func: Callable) -> MethodInfo:
        return parse_func(
            url=url,
            func=func,
            method=method,
            form_body=form_body
        )

    return wrapper


get = partial(rest, method="GET")
post = partial(rest, method="POST")
delete = partial(rest, method="DELETE")
put = partial(rest, method="PUT")



from typing import Any, Dict, Tuple

from pydantic_client.schema.http_request import HttpRequest


class AbstractClient:

    def do_request(
        self, request: HttpRequest,
    ) -> Any:
        raise NotImplementedError

    @staticmethod
    def parse_request(request: HttpRequest) -> Tuple[Dict, Dict]:
        if request.data:
            data = request.data
            json = {}
        else:
            data = {}
            json = request.json_body
        return data, json



import inspect
from typing import Any, Callable, Dict, Type

from pydantic_client.schema.method_info import MethodInfo


def create_response_type(annotations: Dict[str, Any]) -> Type:
    return annotations.pop("return", None)


def parse_func(
    *,
    url: str,
    func: Callable,
    method: str,
    form_body: bool,
):

    spec = inspect.getfullargspec(func)
    annotations = spec.annotations.copy()
    return MethodInfo(
        func=func,
        http_method=method,
        url=url,
        request_type=annotations,
        response_type=create_response_type(annotations),
        form_body=form_body
    )
